@@ -1,7 +1,7 @@
 // qsamplerChannel.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2019, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2020, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, 2008 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -768,11 +768,11 @@ void Channel::appendMessages ( const QString& sText ) const
 }
 
 void Channel::appendMessagesColor (
-	const QString& sText, const QString& sColor ) const
+	const QString& sText, const QColor& rgb ) const
 {
 	MainForm *pMainForm = MainForm::getInstance();
 	if (pMainForm)
-		pMainForm->appendMessagesColor(channelName() + ' ' + sText, sColor);
+		pMainForm->appendMessagesColor(channelName() + ' ' + sText, rgb);
 }
 
 void Channel::appendMessagesText ( const QString& sText ) const
