@@ -1,8 +1,8 @@
 // qsamplerMainForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2019, rncbc aka Rui Nuno Capela. All rights reserved.
-   Copyright (C) 2007,2008,2015 Christian Schoenebeck
+   Copyright (C) 2004-2020, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2007-2019 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -35,6 +35,10 @@
 #include "qsamplerOptionsForm.h"
 #include "qsamplerDeviceStatusForm.h"
 
+#include "qsamplerPaletteForm.h"
+
+#include <QStyleFactory>
+
 #include <QMdiArea>
 #include <QMdiSubWindow>
 
@@ -58,6 +62,8 @@
 #include <QTimer>
 #include <QDateTime>
 
+#include <QElapsedTimer>
+
 #if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 #include <QMimeData>
 #endif
@@ -72,6 +78,11 @@ const WindowFlags WindowCloseButtonHint = WindowFlags(0x08000000);
 #include <gig.h>
 #endif
 
+// Deprecated QTextStreamFunctions/Qt namespaces workaround.
+#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
+#define endl	Qt::endl
+#endif
+
 // Needed for lroundf()
 #include <math.h>
 
@@ -1866,40 +1877,62 @@ void MainForm::viewOptions (void)
 		if (m_pOptions->sMessagesFont.isEmpty() && m_pMessages)
 			m_pOptions->sMessagesFont = m_pMessages->messagesFont().toString();
 		// To track down deferred or immediate changes.
-		const QString sOldServerHost      = m_pOptions->sServerHost;
-		const int     iOldServerPort      = m_pOptions->iServerPort;
-		const int     iOldServerTimeout   = m_pOptions->iServerTimeout;
-		const bool    bOldServerStart     = m_pOptions->bServerStart;
-		const QString sOldServerCmdLine   = m_pOptions->sServerCmdLine;
-		const bool    bOldMessagesLog     = m_pOptions->bMessagesLog;
-		const QString sOldMessagesLogPath = m_pOptions->sMessagesLogPath;
-		const QString sOldDisplayFont     = m_pOptions->sDisplayFont;
-		const bool    bOldDisplayEffect   = m_pOptions->bDisplayEffect;
-		const int     iOldMaxVolume       = m_pOptions->iMaxVolume;
-		const QString sOldMessagesFont    = m_pOptions->sMessagesFont;
-		const bool    bOldKeepOnTop       = m_pOptions->bKeepOnTop;
-		const bool    bOldStdoutCapture   = m_pOptions->bStdoutCapture;
-		const int     bOldMessagesLimit   = m_pOptions->bMessagesLimit;
+		const QString sOldServerHost       = m_pOptions->sServerHost;
+		const int     iOldServerPort       = m_pOptions->iServerPort;
+		const int     iOldServerTimeout    = m_pOptions->iServerTimeout;
+		const bool    bOldServerStart      = m_pOptions->bServerStart;
+		const QString sOldServerCmdLine    = m_pOptions->sServerCmdLine;
+		const bool    bOldMessagesLog      = m_pOptions->bMessagesLog;
+		const QString sOldMessagesLogPath  = m_pOptions->sMessagesLogPath;
+		const QString sOldDisplayFont      = m_pOptions->sDisplayFont;
+		const bool    bOldDisplayEffect    = m_pOptions->bDisplayEffect;
+		const int     iOldMaxVolume        = m_pOptions->iMaxVolume;
+		const QString sOldMessagesFont     = m_pOptions->sMessagesFont;
+		const bool    bOldKeepOnTop        = m_pOptions->bKeepOnTop;
+		const bool    bOldStdoutCapture    = m_pOptions->bStdoutCapture;
+		const int     bOldMessagesLimit    = m_pOptions->bMessagesLimit;
 		const int     iOldMessagesLimitLines = m_pOptions->iMessagesLimitLines;
-		const bool    bOldCompletePath    = m_pOptions->bCompletePath;
-		const bool    bOldInstrumentNames = m_pOptions->bInstrumentNames;
-		const int     iOldMaxRecentFiles  = m_pOptions->iMaxRecentFiles;
-		const int     iOldBaseFontSize    = m_pOptions->iBaseFontSize;
+		const bool    bOldCompletePath     = m_pOptions->bCompletePath;
+		const bool    bOldInstrumentNames  = m_pOptions->bInstrumentNames;
+		const int     iOldMaxRecentFiles   = m_pOptions->iMaxRecentFiles;
+		const int     iOldBaseFontSize     = m_pOptions->iBaseFontSize;
+		const QString sOldCustomStyleTheme = m_pOptions->sCustomStyleTheme;
+		const QString sOldCustomColorTheme = m_pOptions->sCustomColorTheme;
 		// Load the current setup settings.
 		pOptionsForm->setup(m_pOptions);
 		// Show the setup dialog...
 		if (pOptionsForm->exec()) {
 			// Warn if something will be only effective on next run.
+			int iNeedRestart = 0;
 			if (( bOldStdoutCapture && !m_pOptions->bStdoutCapture) ||
-				(!bOldStdoutCapture &&  m_pOptions->bStdoutCapture) ||
-				( bOldKeepOnTop     && !m_pOptions->bKeepOnTop)     ||
+				(!bOldStdoutCapture &&  m_pOptions->bStdoutCapture)) {
+				updateMessagesCapture();
+				++iNeedRestart;
+			}
+			if (( bOldKeepOnTop     && !m_pOptions->bKeepOnTop)     ||
 				(!bOldKeepOnTop     &&  m_pOptions->bKeepOnTop)     ||
 				(iOldBaseFontSize   !=  m_pOptions->iBaseFontSize)) {
-				QMessageBox::information(this,
-					tr("Information"),
-					tr("Some settings may be only effective\n"
-					"next time you start this program."));
-				updateMessagesCapture();
+				++iNeedRestart;
+			}
+			// Check whether restart is needed or whether
+			// custom options maybe set up immediately...
+			if (m_pOptions->sCustomStyleTheme != sOldCustomStyleTheme) {
+				if (m_pOptions->sCustomStyleTheme.isEmpty()) {
+					++iNeedRestart;
+				} else {
+					QApplication::setStyle(
+						QStyleFactory::create(m_pOptions->sCustomStyleTheme));
+				}
+			}
+			if (m_pOptions->sCustomColorTheme != sOldCustomColorTheme) {
+				if (m_pOptions->sCustomColorTheme.isEmpty()) {
+					++iNeedRestart;
+				} else {
+					QPalette pal;
+					if (PaletteForm::namedPalette(
+							&m_pOptions->settings(), m_pOptions->sCustomColorTheme, pal))
+						QApplication::setPalette(pal);
+				}
 			}
 			// Check wheather something immediate has changed.
 			if (( bOldMessagesLog && !m_pOptions->bMessagesLog) ||
@@ -1927,6 +1960,13 @@ void MainForm::viewOptions (void)
 				(!bOldMessagesLimit &&  m_pOptions->bMessagesLimit) ||
 				(iOldMessagesLimitLines !=  m_pOptions->iMessagesLimitLines))
 				updateMessagesLimit();
+			// Show restart needed message...
+			if (iNeedRestart > 0) {
+				QMessageBox::information(this,
+					tr("Information"),
+					tr("Some settings may be only effective\n"
+					"next time you start this program."));
+			}
 			// And now the main thing, whether we'll do client/server recycling?
 			if ((sOldServerHost != m_pOptions->sServerHost) ||
 				(iOldServerPort != m_pOptions->iServerPort) ||
@@ -2454,7 +2494,7 @@ void MainForm::updateMaxVolume (void)
 // QSampler::MainForm -- Messages window form handlers.
 
 // Messages output methods.
-void MainForm::appendMessages( const QString& s )
+void MainForm::appendMessages ( const QString& s )
 {
 	if (m_pMessages)
 		m_pMessages->appendMessages(s);
@@ -2462,26 +2502,26 @@ void MainForm::appendMessages( const QString& s )
 	statusBar()->showMessage(s, 3000);
 }
 
-void MainForm::appendMessagesColor( const QString& s, const QString& c )
+void MainForm::appendMessagesColor ( const QString& s, const QColor& rgb )
 {
 	if (m_pMessages)
-		m_pMessages->appendMessagesColor(s, c);
+		m_pMessages->appendMessagesColor(s, rgb);
 
 	statusBar()->showMessage(s, 3000);
 }
 
-void MainForm::appendMessagesText( const QString& s )
+void MainForm::appendMessagesText ( const QString& s )
 {
 	if (m_pMessages)
 		m_pMessages->appendMessagesText(s);
 }
 
-void MainForm::appendMessagesError( const QString& sText )
+void MainForm::appendMessagesError ( const QString& s )
 {
 	if (m_pMessages)
 		m_pMessages->show();
 
-	appendMessagesColor(sText.simplified(), "#ff0000");
+	appendMessagesColor(s.simplified(), Qt::red);
 
 	// Make it look responsive...:)
 	QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
@@ -2494,7 +2534,7 @@ void MainForm::appendMessagesError( const QString& sText )
 		QMessageBox mbox(this);
 		mbox.setIcon(QMessageBox::Critical);
 		mbox.setWindowTitle(sTitle);
-		mbox.setText(sText);
+		mbox.setText(s);
 		mbox.setStandardButtons(QMessageBox::Cancel);
 		QCheckBox cbox(tr("Don't show this again"));
 		cbox.setChecked(false);
@@ -2792,6 +2832,17 @@ void MainForm::timerSlot (void)
 				}
 			}
 		}
+
+	#if CONFIG_LSCP_CLIENT_CONNECTION_LOST
+		// If we lost connection to server: Try to automatically reconnect if we
+		// did not start the server.
+		//
+		// TODO: If we started the server, then we might inform the user that
+		// the server probably crashed and asking user ONCE whether we should
+		// restart the server.
+		if (lscp_client_connection_lost(m_pClient) && !m_pServer)
+			startAutoReconnectClient();
+	#endif // CONFIG_LSCP_CLIENT_CONNECTION_LOST
 	}
 
 	// Register the next timer slot.
@@ -2918,9 +2969,9 @@ void MainForm::stopServer ( bool bInteractive )
 
 	// Give it some time to terminate gracefully and stabilize...
 	if (bGraceWait) {
-		QTime t;
-		t.start();
-		while (t.elapsed() < QSAMPLER_TIMER_MSECS)
+		QElapsedTimer timer;
+		timer.start();
+		while (timer.elapsed() < QSAMPLER_TIMER_MSECS)
 			QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
 	}
 }
@@ -2950,9 +3001,9 @@ void MainForm::processServerExit (void)
 			// Force final server shutdown...
 			m_pServer->kill();
 			// Give it some time to terminate gracefully and stabilize...
-			QTime t;
-			t.start();
-			while (t.elapsed() < QSAMPLER_TIMER_MSECS)
+			QElapsedTimer timer;
+			timer.start();
+			while (timer.elapsed() < QSAMPLER_TIMER_MSECS)
 				QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
 		}
 		// Force final server shutdown...
@@ -2990,7 +3041,7 @@ lscp_status_t qsampler_client_callback ( lscp_client_t */*pClient*/,
 
 
 // Start our almighty client...
-bool MainForm::startClient (void)
+bool MainForm::startClient (bool bReconnectOnly)
 {
 	// Have it a setup?
 	if (m_pOptions == nullptr)
@@ -3011,9 +3062,15 @@ bool MainForm::startClient (void)
 		// Is this the first try?
 		// maybe we need to start a local server...
 		if ((m_pServer && m_pServer->state() == QProcess::Running)
-			|| !m_pOptions->bServerStart) {
-			appendMessagesError(
-				tr("Could not connect to server as client.\n\nSorry."));
+			|| !m_pOptions->bServerStart || bReconnectOnly)
+		{
+			// if this method is called from autoReconnectClient()
+			// then don't bother user with an error message...
+			if (!bReconnectOnly) {
+				appendMessagesError(
+					tr("Could not connect to server as client.\n\nSorry.")
+				);
+			}
 		} else {
 			startServer();
 		}
@@ -3142,6 +3199,22 @@ void MainForm::stopClient (void)
 }
 
 
+void MainForm::startAutoReconnectClient (void)
+{
+	stopClient();
+	appendMessages(tr("Trying to reconnect..."));
+	QTimer::singleShot(QSAMPLER_TIMER_MSECS, this, SLOT(autoReconnectClient()));
+}
+
+
+void MainForm::autoReconnectClient (void)
+{
+	const bool bSuccess = startClient(true);
+	if (!bSuccess)
+		QTimer::singleShot(QSAMPLER_TIMER_MSECS, this, SLOT(autoReconnectClient()));
+}
+
+
 // Channel strip activation/selection.
 void MainForm::activateStrip ( QMdiSubWindow *pMdiSubWindow )
 {
