@@ -1,8 +1,8 @@
 // qsamplerMainForm.h
 //
 /****************************************************************************
-   Copyright (C) 2004-2019, rncbc aka Rui Nuno Capela. All rights reserved.
-   Copyright (C) 2007,2008,2015 Christian Schoenebeck
+   Copyright (C) 2004-2020, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2007-2019 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -64,11 +64,11 @@ public:
 
 	QString sessionName(const QString& sFilename);
 
-	void appendMessages(const QString& sText);
-	void appendMessagesColor(const QString& sText, const QString& sColor);
-	void appendMessagesText(const QString& sText);
-	void appendMessagesError(const QString& sText);
-	void appendMessagesClient(const QString& sText);
+	void appendMessages(const QString& s);
+	void appendMessagesColor(const QString& s, const QColor& rgb);
+	void appendMessagesText(const QString& s);
+	void appendMessagesError(const QString& s);
+	void appendMessagesClient(const QString& s);
 
 	ChannelStrip *createChannelStrip(Channel *pChannel);
 	void destroyChannelStrip(ChannelStrip *pChannelStrip);
@@ -123,6 +123,7 @@ protected slots:
 	void timerSlot();
 	void readServerStdout();
 	void processServerExit();
+	void autoReconnectClient();
 
 	void handle_sigusr1();
 	void handle_sigterm();
@@ -165,8 +166,9 @@ protected:
 	void stopSchedule();
 	void startServer();
 	void stopServer(bool bInteractive = false);
-	bool startClient();
+	bool startClient(bool bReconnectOnly = false);
 	void stopClient();
+	void startAutoReconnectClient();
 
 private:
 
