@@ -1,7 +1,7 @@
 // qsamplerPaletteForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2019, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2020, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -78,6 +78,9 @@ static struct
 	{ "LinkVisited",     QPalette::LinkVisited     },
 	{ "ToolTipBase",     QPalette::ToolTipBase     },
 	{ "ToolTipText",     QPalette::ToolTipText     },
+#if QT_VERSION >= QT_VERSION_CHECK(5, 12, 0)
+	{ "PlaceholderText", QPalette::PlaceholderText },
+#endif
 	{ "NoRole",          QPalette::NoRole          },
 
 	{  nullptr,          QPalette::NoRole          }
@@ -115,9 +118,6 @@ PaletteForm::PaletteForm ( QWidget *parent, const QPalette& pal )
 	m_ui.paletteView->setColumnHidden(2, true);
 	m_ui.paletteView->setColumnHidden(3, true);
 
-	QObject::connect(m_ui.nameCombo,
-		SIGNAL(activated(const QString&)),
-		SLOT(nameComboActivated(const QString&)));
 	QObject::connect(m_ui.nameCombo,
 		SIGNAL(editTextChanged(const QString&)),
 		SLOT(nameComboChanged(const QString&)));
@@ -229,18 +229,14 @@ QSettings *PaletteForm::settings (void) const
 }
 
 
-void PaletteForm::nameComboActivated ( const QString& name )
-{
-	setPaletteName(name);
-}
-
-
 void PaletteForm::nameComboChanged ( const QString& name )
 {
-	if (m_dirtyCount > 0 || m_ui.nameCombo->findText(name) < 0)
+	if (m_dirtyCount > 0 || m_ui.nameCombo->findText(name) < 0) {
 		updateDialogButtons();
-	else
+	} else {
 		setPaletteName(name);
+		++m_dirtyTotal;
+	}
 }
 
 
@@ -516,7 +512,9 @@ bool PaletteForm::namedPalette (
 	QSettings *settings, const QString& name, QPalette& pal, bool fixup )
 {
 	int result = 0;
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
 	uint mask = pal.resolve();
+#endif
 
 	// Custom color themes...
 	if (name == "Wonton Soup") {
@@ -577,7 +575,9 @@ bool PaletteForm::namedPalette (
 		pal.setColor(QPalette::Active,   QPalette::LinkVisited, QColor(64, 128, 255));
 		pal.setColor(QPalette::Inactive, QPalette::LinkVisited, QColor(64, 128, 255));
 		pal.setColor(QPalette::Disabled, QPalette::LinkVisited, QColor(54, 76, 119));
+	#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
 		mask = 0;
+	#endif
 		++result;
 	}
 	else
@@ -639,7 +639,9 @@ bool PaletteForm::namedPalette (
 		pal.setColor(QPalette::Active,   QPalette::LinkVisited, QColor(230, 100, 230));
 		pal.setColor(QPalette::Inactive, QPalette::LinkVisited, QColor(230, 100, 230));
 		pal.setColor(QPalette::Disabled, QPalette::LinkVisited, QColor(74, 34, 74));
+	#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
 		mask = 0;
+	#endif
 		++result;
 	}
 	else
@@ -657,7 +659,9 @@ bool PaletteForm::namedPalette (
 				pal.setColor(QPalette::Active,   cr, QColor(clist.at(0)));
 				pal.setColor(QPalette::Inactive, cr, QColor(clist.at(1)));
 				pal.setColor(QPalette::Disabled, cr, QColor(clist.at(2)));
+			#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
 				mask &= ~(1 << int(cr));
+			#endif
 				++result;
 			}
 		}
@@ -696,7 +700,9 @@ bool PaletteForm::namedPalette (
 		++result;
 	}
 
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
 	pal.resolve(mask);
+#endif
 	return (result > 0);
 }
 
@@ -967,22 +973,22 @@ bool PaletteForm::PaletteModel::setData (
 		if (m_generate) {
 			m_palette.setBrush(QPalette::Inactive, cr, color);
 			switch (cr) {
-				case QPalette::Foreground:
+				case QPalette::WindowText:
 				case QPalette::Text:
 				case QPalette::ButtonText:
 				case QPalette::Base:
 					break;
 				case QPalette::Dark:
-					m_palette.setBrush(QPalette::Disabled, QPalette::Foreground, color);
+					m_palette.setBrush(QPalette::Disabled, QPalette::WindowText, color);
 					m_palette.setBrush(QPalette::Disabled, QPalette::Dark, color);
 					m_palette.setBrush(QPalette::Disabled, QPalette::Text, color);
 					m_palette.setBrush(QPalette::Disabled, QPalette::ButtonText, color);
 					index_begin = PaletteModel::index(0, 0);
 					index_end = PaletteModel::index(m_nrows - 1, 3);
 					break;
-				case QPalette::Background:
+				case QPalette::Window:
 					m_palette.setBrush(QPalette::Disabled, QPalette::Base, color);
-					m_palette.setBrush(QPalette::Disabled, QPalette::Background, color);
+					m_palette.setBrush(QPalette::Disabled, QPalette::Window, color);
 					index_begin = PaletteModel::index(QPalette::Base, 0);
 					break;
 				case QPalette::Highlight:
@@ -1261,7 +1267,7 @@ PaletteForm::ColorEditor::ColorEditor ( QWidget *parent )
 	: QWidget(parent)
 {
 	QLayout *layout = new QHBoxLayout(this);
-	layout->setMargin(0);
+	layout->setContentsMargins(0, 0, 0, 0);
 	m_button = new PaletteForm::ColorButton(this);
 	layout->addWidget(m_button);
 	QObject::connect(m_button,
@@ -1307,7 +1313,7 @@ PaletteForm::RoleEditor::RoleEditor ( QWidget *parent )
 	m_edited = false;
 
 	QHBoxLayout *layout = new QHBoxLayout(this);
-	layout->setMargin(0);
+	layout->setContentsMargins(0, 0, 0, 0);
 	layout->setSpacing(0);
 
 	m_label = new QLabel(this);
